package no.stud.bolts;

import java.util.Map;

import backtype.storm.task.OutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.BasicOutputCollector;
import backtype.storm.topology.IBasicBolt;
import backtype.storm.topology.IRichBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Tuple;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import no.stud.util.CustomConfig;
import no.stud.util.DistributionType;

import backtype.storm.topology.base.BaseRichBolt;

/**
 * Dumps strings followed of a newline to a CP-1252 (Windows 7) or UTF-8 format(Ubuntu Linux)
 */
public class BinFileDumpBolt implements IRichBolt {
	Integer id;
	String name;
	FileOutputStream fileOutputStream;
	BufferedOutputStream bufferedOutputStream;
	DataOutputStream dataOutputStream;
	int PointDimensions;


	public void cleanup() {
		try {
			fileOutputStream.close();
			bufferedOutputStream.close();
			dataOutputStream.close();
		} catch (IOException e) { e.printStackTrace(); }
	}


	public void declareOutputFields(OutputFieldsDeclarer declarer) {}


	public void prepare(Map stormConf, TopologyContext context, OutputCollector collector) {
		this.name = context.getThisComponentId();
		this.id = context.getThisTaskId();
		this.PointDimensions = ((Long)stormConf.get(CustomConfig.PointDimensions)).intValue();

		DistributionType distType = DistributionType.toType(((Long)stormConf.get(CustomConfig.GeneratorType)).intValue());
		String filePath = "/mnt/storm/new/"+PointDimensions+"d_"+distType.getId()+"t_"+name+"_";

		try {
			this.fileOutputStream = new FileOutputStream(filePath);
			this.bufferedOutputStream = new BufferedOutputStream(fileOutputStream);
			this.dataOutputStream = new DataOutputStream(bufferedOutputStream);
		} catch (IOException e) { e.printStackTrace(); }

	}


	public void execute(Tuple input) {
		double[] in = (double[]) input.getValue(0);
		if(in != null) if(in.length>0) {
			try {
				for (int i=0; i <this.PointDimensions; i++)
					dataOutputStream.writeDouble(in[i]);
				//bw.flush();		// Simply for avoiding write of hafe lines on shutdown. (is not hurting performance have a look at time execute on bolt)
			} catch (IOException e) { e.printStackTrace(); }
		}
	}


	public Map<String, Object> getComponentConfiguration() {
		// TODO Auto-generated method stub
		return null;
	}
}
