package no.stud.bolts;

import java.util.Map;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.BasicOutputCollector;
import backtype.storm.topology.IBasicBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;

import no.stud.util.PointCompare;
import java.util.ArrayList;



public class FilterForwardSkylineBolt implements IBasicBolt {
	ArrayList<Double[]> skyList, remList;

	/**
	 */
	public void cleanup() {}

	/**
	 * On create 
	 */
	public void prepare(Map stormConf, TopologyContext context) {
		skyList = new ArrayList<Double[]>();		// A ArrayList to hold the skyline.
	}

	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(new Fields("multi-point"));
	}


	public void execute(Tuple input, BasicOutputCollector collector) {
		//	Working serialization
		double[] in = (double[]) input.getValue(0);
		if(in != null) if(in.length>0) {

			// new remove list.
			remList = new ArrayList<Double[]>();

			// As we will do the comparation more times, hopefully save some overhead on converting to Double
			Double[] recivedPoint = new Double[in.length];
			for (int i=0; i<in.length; i++)
			{
				recivedPoint[i] = Double.valueOf(in[i]);
			}

			for(int x=0; x<skyList.size(); x++)
			{
				// If we dominate one, add point to list for removal, and check next point. 
				if(PointCompare.isDominatedOfSimple(skyList.get(x), recivedPoint)) {
					remList.add(skyList.get(x));
					// we can not become dominated when we are dominating a point.
					continue;
				}
				// we can not become dominated if a point is dominated.
				if(remList.size() > 0) continue;

				// if we are dominated, toss the point aka just return.
				if(PointCompare.isDominatedOfSimple(recivedPoint, skyList.get(x))) return;

			}

			// if we got so far, no one dominated us, or skyline is empty.
			// remove the ones we dominated, if any.
			skyList.removeAll(remList);
			remList = null;
			// Remove the oldest skyline point if skyline have hit maxSize of 100.
			if(skyList.size() >= 100) {
				skyList.remove(0);
			}
			// add recivedPoint to the skyline
			skyList.add(recivedPoint);
			collector.emit(new Values(in));
		}
	}

	@Override
	public Map<String, Object> getComponentConfiguration() {
		// TODO Auto-generated method stub
		return null;
	}

}