package no.stud.bolts;

import java.util.Map;

import backtype.storm.Config;
import backtype.storm.Constants;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.BasicOutputCollector;
import backtype.storm.topology.IBasicBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Tuple;

import no.stud.util.PointCompare;
import java.util.ArrayList;



@SuppressWarnings("serial")
public class SimpleSkylineBolt implements IBasicBolt {
	Integer id;		// getThisTaskId, unique number
	String name;	// getThisComponentId, bolt type
	ArrayList<Double[]> skyList, remList;
	long tickTupleCount = 0;

	public void cleanup() {}

	/**
	 * On create 
	 */
	public void prepare(Map stormConf, TopologyContext context) {
		this.name = context.getThisComponentId();		
		this.id = context.getThisTaskId();		 
		skyList = new ArrayList<Double[]>();			// A ArrayList to hold the skyline.
		System.out.println("prepare() is ran.");		// Debug output to show that bolt is initialized.

	}

	public void declareOutputFields(OutputFieldsDeclarer declarer) {}

	/***
	 * Thanks to M. Noll for the isTickTuple function.
	 * http://www.michael-noll.com/blog/2013/01/18/implementing-real-time-trending-topics-in-storm/
	 */
	private static boolean isTickTuple(Tuple tuple) {
		return tuple.getSourceComponent().equals(Constants.SYSTEM_COMPONENT_ID)
				&& tuple.getSourceStreamId().equals(Constants.SYSTEM_TICK_STREAM_ID);
	}

	public void execute(Tuple input, BasicOutputCollector collector) {
		// Do a periodic report of the total Skyline size, every time tick is received.
		if (isTickTuple(input)) {
			tickTupleCount++;
			System.out.println(tickTupleCount*10+"s. "+skyList.size()+" points.");
		} else {
			// Normal tuple
			//	Working serialization with double array
			double[] in = (double[]) input.getValue(0);
			if(in != null) if(in.length>0) {

				// clean the remove list.
				remList = new ArrayList<Double[]>();

				// As we will do the comparation more times, save some overhead on converting to Double
				Double[] recivedPoint = new Double[in.length];
				for (int i=0; i<in.length; i++)
				{
					recivedPoint[i] = Double.valueOf(in[i]);
				}


				boolean dominated=false;
				outerloop:
					for(int x=0; x<skyList.size(); x++)
					{
						// if we are dominated, toss the point aka just return.
						if(PointCompare.isDominatedOfSimple(recivedPoint, skyList.get(x))){ 
							dominated = true;
							break outerloop;
						}

						// if we dominate one, add us to the list and add them to list of removal
						if(PointCompare.isDominatedOfSimple(skyList.get(x), recivedPoint))remList.add(skyList.get(x));
					}
				// if we got so far, no one dominated us, or skyline is empty.

				// remove the ones we dominated, if any.
				skyList.removeAll(remList);
				// add us self to the list. If we ain't dominated
				if(!dominated)
					skyList.add(recivedPoint);

				// dereference for gbc to be able to re claim memory
				remList=null;
				in = null;
			}	
		}
	}

	public Map<String, Object> getComponentConfiguration() {
		Config configObject = new Config();
		configObject.put(Config.TOPOLOGY_TICK_TUPLE_FREQ_SECS, 10);
		return configObject;
	}
}
