package no.stud.spouts;
import java.util.Map;

import backtype.storm.spout.SpoutOutputCollector;
import backtype.storm.topology.IRichSpout;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.task.TopologyContext;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;
import backtype.storm.utils.Utils;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;

import no.stud.util.CustomConfig;
import no.stud.util.DistributionType;

public class BinFileReadSpout implements IRichSpout {
	Integer id, taskIndex;
	String name;
	SpoutOutputCollector collector;		

	FileInputStream fileInputStream = null;
	BufferedInputStream bufferedInputStream = null;
	DataInputStream dataInputStream = null;

	double[] in;

	int PointDimensions;
	private boolean completed = false;
	private boolean sleepEnabled = true;

	// Using a toggle to sleep every Xnd run.	This value is overridden by customConfig if defined.
	int toggleSleepValue = 10;
	int toggleSleepTime = 1;
	int toggleCount = 0;


	/*If reliability is on, and we get a ack. Remove it from send queue.*/
	public void ack(Object msgId) {
		// 		System.out.println("OK:"+msgId);
		//		Action to remove from queue.
	}

	public void close() {
		try {
			dataInputStream.close();
			bufferedInputStream.close();
			fileInputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}	
	}

	public void fail(Object msgId) {
		System.out.println("FAIL:"+msgId);
	}


	/**
	 * The only thing that the methods will do It is emit each generated random until limit reached 
	 */
	public void nextTuple() {
		// Short: Avoid deadloop, filling the buffers and crashing.
		//
		// Without sleep, would we always have work until buffers are full.
		// That causes also deadloop resource starvation in addition to 
		// communication breakdown in ZMQ, in random spouts\bolts that tries
		// to communicate with it causing random crashes.
		if(sleepEnabled)
			if(toggleCount++ >= toggleSleepValue) {
				Utils.sleep(toggleSleepTime);
				toggleCount = 0;
			}


		if(completed){
			Utils.sleep(10);	// Keep a low value, we still wanna be able to ack and replay, if failed.
		} else {
			try {
				for(int i=0; i<this.PointDimensions; i++) {
					in[i] = dataInputStream.readDouble();
				}
				this.collector.emit( /*broadcastToStreamId,*/ new Values(in));

			} catch (IOException e) {
				completed=true;
				e.printStackTrace();
			}	
		}
	}

	/**
	 * Init the spout, create the CorrelatedGenerator and get the collector object
	 */
	public void open(Map stormConf, TopologyContext context, SpoutOutputCollector collector) {
		this.name = context.getThisComponentId();
		this.id = context.getThisTaskId();
		this.taskIndex = context.getThisTaskIndex();
		this.collector = collector;
		this.PointDimensions = ((Long)stormConf.get(CustomConfig.PointDimensions)).intValue();

		try {
			this.toggleSleepValue = ((Long)stormConf.get(CustomConfig.ToggleSleepValue)).intValue();
			if (toggleSleepValue == -1)sleepEnabled=false;
		} catch (Exception e1) {
			// TODO No override exist.
		}
		try {
			this.toggleSleepTime = ((Long)stormConf.get(CustomConfig.ToggleSleepTime)).intValue();
		} catch (Exception e1) {
			// using default 1ms.
		}
		DistributionType distType = DistributionType.toType(((Long)stormConf.get(CustomConfig.GeneratorType)).intValue());

		String strFilePath = "/mnt/storm/new/"+PointDimensions+"d_"+distType.getId()+"t_FileDump_"+((this.taskIndex%3)+1);
		try {
			//create FileInputStream object
			fileInputStream = new FileInputStream(strFilePath);
		} catch (Exception e) {	
		}
		finally {}

		// to reuse the array we fill with read doubles and use to send it.
		in = new double[PointDimensions];

		try
		{
			if(fileInputStream == null){
				System.err.println("Probing or static input filepath failed.");
				throw new IOException();
			}

			this.bufferedInputStream = new BufferedInputStream(fileInputStream);
			this.dataInputStream = new DataInputStream(bufferedInputStream);

		} catch (IOException e) {
			completed=true;
			e.printStackTrace();
			// throw new IllegalArgumentException("Forcing a shutdown of topology, input din't exist.");
		}
	}

	/**
	 * Declare the output field(s) ""
	 */
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(new Fields("multi-point"));
	}

	@Override
	public void activate() {
		// TODO Auto-generated method stub

	}

	@Override
	public void deactivate() {
		// TODO Auto-generated method stub

	}

	@Override
	public Map<String, Object> getComponentConfiguration() {
		// TODO Auto-generated method stub
		return null;
	}
}
