package no.stud.spouts;

import java.util.Map;
import backtype.storm.spout.SpoutOutputCollector;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.topology.base.BaseRichSpout;
import backtype.storm.task.TopologyContext;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;
import backtype.storm.utils.Utils;

import no.stud.util.CustomConfig;
import no.stud.util.DistributionType;

import no.stud.spouts.datagenerators.*;

public class GenerateDatasetSpout extends BaseRichSpout {
	
	// Using a toggle to sleep every Xnd time.	
	int toggleSleepValue = 15;
	int toggleCount = 0;
	
	// Storms standard output collector
	SpoutOutputCollector collector;
	
	// Our dynamically allocated datagenerator
	DistributionGenerator dg;

	/*If reliability is on, and we get a ack. Remove it from send queue.*/
	public void ack(Object msgId) {
// 		System.out.println("OK:"+msgId);
//		Action to remove from queue.
	}
	
	/* Action to do before bolt shutdown. */
	public void close() {}
	
	/*If reliability is on, and we do not get a ack. Put message back on queue.*/
	public void fail(Object msgId) {
// 		System.out.println("FAIL:"+msgId);
	}

	/**
	 * The only thing this methods will do It is emit a generated point. 
	 */
	public void nextTuple() {
		// Short: Avoid deadloop, filling the buffers and crashing.
		//
		// Without sleep, would we always have work until buffers are full.
		// That causes also deadloop resource starvation in addition to 
		// communication breakdown in ZMQ, in random spouts\bolts that tries
		// to communicate with it causing random crashes.
		if(++toggleCount >= toggleSleepValue) {
			Utils.sleep(1);
			toggleCount = 0;
		}
		
		// Get the next point from the generator.
		double[] doubleA = dg.nextPoint();
		// Emit the next tuple.
		this.collector.emit( /*broadcastToStreamId,*/ new Values(doubleA) /*, messageIdentificator*/);
	}

	/**
	 * Init the spout, create the CorrelatedGenerator and get the collector object
	 */
	public void open(Map conf, TopologyContext context, SpoutOutputCollector collector) {
		// Set the output collector.
		this.collector = collector;
		
		// Read the config values from the serialized storm config.
		int DimensionMaxValue = ((Long)conf.get(CustomConfig.DimensionMaxValue)).intValue();
		int PointDimensions = ((Long)conf.get(CustomConfig.PointDimensions)).intValue();
		DistributionType distType = DistributionType.toType(((Long)conf.get(CustomConfig.GeneratorType)).intValue());
		
		// Initialize the correct distribution generator with the values from the config.
		switch (distType) {
			case ANTICORRELATED:
				this.dg = new AntiCorrelatedGenerator(DimensionMaxValue, PointDimensions); 
				break;
			case CORRELATED:
				this.dg = new CorrelatedGenerator(DimensionMaxValue, PointDimensions); 
				break;
			case UNIFORM:
				this.dg = new UniformGenerator(DimensionMaxValue, PointDimensions); 
				break;
			default:
				break;
		}
	}

	/**
	 * Declare the output field(s) ""
	 */
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(new Fields("multi-point"));
	}
}
